"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const schematics_1 = require("@angular-devkit/schematics");
const core_1 = require("@angular-devkit/core");
const schematics_2 = require("@angular-devkit/schematics");
const ng_module_utils_1 = require("../utils/ng-module-utils");
const JSON5 = require("json5");
const crudModelUtils = require("../utils/crud-model-utils");
const config_1 = require("@schematics/angular/utility/config");
const parse_name_1 = require("@schematics/angular/utility/parse-name");
const find_module_1 = require("../schematics-angular-utils/find-module");
function setupOptions(options, host) {
    const workspace = config_1.getWorkspace(host);
    if (!options.project) {
        options.project = Object.keys(workspace.projects)[0];
    }
    const project = workspace.projects[options.project];
    if (options.path === undefined) {
        const projectDirName = project.projectType === 'application' ? 'app' : 'lib';
        options.path = `/${project.root}/src/${projectDirName}`;
    }
    const parsedPath = parse_name_1.parseName(options.path, options.name);
    options.name = parsedPath.name;
    options.path = parsedPath.path;
}
function default_1(options) {
    return (host, context) => {
        setupOptions(options, host);
        options.module = find_module_1.findModuleFromOptions(host, options, true) || '';
        const modelFile = `/${options.path}/${options.model}`;
        const modelBuffer = host.read(modelFile);
        if (modelBuffer === null) {
            throw new schematics_1.SchematicsException(`Model file ${options.model} does not exist.`);
        }
        const modelJson = modelBuffer.toString('utf-8');
        const model = JSON5.parse(modelJson);
        const templateSource = schematics_2.apply(schematics_2.url('./files'), [
            schematics_2.template(Object.assign({}, core_1.strings, options, crudModelUtils, { model })),
            schematics_2.move(options.path || '')
        ]);
        const rule = schematics_1.chain([
            schematics_2.branchAndMerge(schematics_1.chain([
                schematics_1.mergeWith(templateSource),
                ng_module_utils_1.addImportToParentModule(options)
            ])),
        ]);
        return rule(host, context);
    };
}
exports.default = default_1;
//# sourceMappingURL=index.js.map