"use strict";
// Option A: Directly referencing the private APIs
// import { ModuleOptions, buildRelativePath } from "@schematics/angular/utility/find-module";
// import { Rule, Tree, SchematicsException } from "@angular-devkit/schematics";
// import { dasherize, classify } from "@angular-devkit/core";
// import { addDeclarationToModule, addExportToModule } from "@schematics/angular/utility/ast-utils";
// import { InsertChange } from "@schematics/angular/utility/change";
Object.defineProperty(exports, "__esModule", { value: true });
// Option B: Using a fork of the private APIs b/c they can change
const schematics_1 = require("@angular-devkit/schematics");
const add_to_module_context_1 = require("./add-to-module-context");
const ts = require("typescript");
const core_1 = require("@angular-devkit/core");
// Referencing forked and copied private APIs 
const find_module_1 = require("../schematics-angular-utils/find-module");
const ast_utils_1 = require("../schematics-angular-utils/ast-utils");
const change_1 = require("../schematics-angular-utils/change");
function addImportToParentModule(options) {
    return (host) => {
        addImport(host, options);
        return host;
    };
}
exports.addImportToParentModule = addImportToParentModule;
function createAddImportToModuleContext(host, options) {
    const result = new add_to_module_context_1.AddImportToModuleContext();
    if (!options.module) {
        throw new schematics_1.SchematicsException(`Module not found.`);
    }
    const text = host.read(options.module);
    if (text === null) {
        throw new schematics_1.SchematicsException(`File ${options.module} does not exist.`);
    }
    const sourceText = text.toString('utf-8');
    result.source = ts.createSourceFile(options.module, sourceText, ts.ScriptTarget.Latest, true);
    const moduleToImportPath = `/${options.sourceDir}/${options.path}/`
        + core_1.strings.dasherize(options.name)
        + '.module';
    result.relativePath = find_module_1.buildRelativePath(options.module, moduleToImportPath);
    result.classifiedName = core_1.strings.classify(`${options.name}Module`);
    return result;
}
function addImport(host, options) {
    const context = createAddImportToModuleContext(host, options);
    const modulePath = options.module || '';
    const declarationChanges = ast_utils_1.addImportToModule(context.source, modulePath, context.classifiedName, context.relativePath);
    const declarationRecorder = host.beginUpdate(modulePath);
    for (const change of declarationChanges) {
        if (change instanceof change_1.InsertChange) {
            declarationRecorder.insertLeft(change.pos, change.toAdd);
        }
    }
    host.commitUpdate(declarationRecorder);
}
;
//# sourceMappingURL=ng-module-utils.js.map